@@ -1,4 +1,7 @@
-import tktooltip
+try:
+    import tktooltip
+except ImportError:
+    pass
 import tkinter as tk
 import tkinter.ttk as ttk
 import tkinter.tix as tix
@@ -6,31 +9,124 @@ from tkdev.devresize import DevResize
 from tkdev.devicon import Icon_Empty
 from ctypes import windll
 from os import environ
-from win32gui import *
-from win32con import *
-from PySide6.QtCore import Qt
+
+try:
+    from win32gui import *
+    from win32con import *
+    from win32api import *
+except ImportError:
+    pass
+
+windll.shcore.SetProcessDpiAwareness(1)
+ScaleFactor = windll.shcore.GetScaleFactorForDevice(0)
+
+taskbar_height = GetMonitorInfo(MonitorFromPoint((0, 0))).get("Monitor")[3] - \
+                 GetMonitorInfo(MonitorFromPoint((0, 0))).get("Work")[3]
 
 
-windll.user32.SetProcessDPIAware()
+def window_pos_bottom_right(window: tk.Tk, padx: int = 15, pady: int = 15):
+    window.after(1, lambda: window.geometry(
+        f"+{window.winfo_screenwidth() - window.winfo_width() - padx - 10}+{window.winfo_screenheight() - taskbar_height - window.winfo_height() - pady - 35}"))
+
+
+def window_pos_bottom_left(window: tk.Tk, padx: int = 15, pady: int = 15):
+    window.after(1, lambda: window.geometry(
+        f"+{padx}+{window.winfo_screenheight() - taskbar_height - window.winfo_height() - pady - 35}"))
+
+
+def window_pos_top_right(window: tk.Tk, padx: int = 15, pady: int = 15):
+    window.after(1, lambda: window.geometry(
+        f"+{window.winfo_screenwidth() - window.winfo_width() - padx - 10}+{pady}"))
+
+
+def window_pos_top_left(window: tk.Tk, padx: int = 15, pady: int = 15):
+    window.after(1, lambda: window.geometry(
+        f"+{padx}+{pady}"))
+
+
+def window_sizebox(window: tk.Tk):
+    def sizebox():
+        hwnd = GetParent(window.winfo_id())
+        res = SetWindowLong(hwnd, GWL_STYLE, GetWindowLong(hwnd, GWL_STYLE) & ~WS_SIZEBOX)
+
+    window.after(1, sizebox)
+
+
+def window_maxbox(window: tk.Tk):
+    def max():
+        hwnd = GetParent(window.winfo_id())
+        res = SetWindowLong(hwnd, GWL_STYLE, GetWindowLong(hwnd, GWL_STYLE) & ~WS_MAXIMIZEBOX)
+
+    window.after(1, max)
+
+
+def window_minbox(window: tk.Tk):
+    def min():
+        hwnd = GetParent(window.winfo_id())
+        res = SetWindowLong(hwnd, GWL_STYLE, GetWindowLong(hwnd, GWL_STYLE) & ~WS_MINIMIZEBOX)
+
+    window.after(1, min)
+
+
+def window_grayed_closebox(window: tk.Tk):
+    def close():
+        hwnd = GetParent(window.winfo_id())
+        res = EnableMenuItem(GetSystemMenu(hwnd, False), SC_CLOSE, MF_BYCOMMAND | MF_GRAYED)
+
+    window.after(1, max)
+
+
+def window_enabled_closebox(window: tk.Tk):
+    def close():
+        hwnd = GetParent(window.winfo_id())
+        res = EnableMenuItem(GetSystemMenu(hwnd, False), SC_CLOSE, MF_BYCOMMAND | MF_ENABLED)
+
+    window.after(1, max)
+
+
+def window_popup(window: tk.Tk):
+    def popup():
+        hwnd = GetParent(window.winfo_id())
+        res = SetWindowLong(hwnd, GWL_STYLE, GetWindowLong(hwnd, GWL_STYLE) & ~WS_POPUPWINDOW)
+
+    window.after(1, popup)
+
+
+def window_custom_taskbar(window: tk.Tk):
+    def over():
+        hwnd = GetParent(window.winfo_id())
+        res = SetWindowLong(hwnd, GWL_STYLE, GetWindowLong(hwnd, GWL_STYLE) & ~WS_OVERLAPPEDWINDOW)
+
+    window.after(1, over)
+    window.update()
+
+
+def window_custom_border_taskbar(window: tk.Tk):
+    def custom():
+        hwnd = GetParent(window.winfo_id())
+        res = SetWindowLong(hwnd, GWL_STYLE, GetWindowLong(hwnd, GWL_STYLE) & ~WS_BORDER)
+
+    window.after(1, custom)
+    window.update()
 
 
 def window_add_taskbar(window: tk.Tk):
     hwnd = GetParent(window.winfo_id())
-    style = GetWindowLong(hwnd, GWL_EXSTYLE)
-    style = style & ~WS_EX_TOOLWINDOW
-    style = style | WS_EX_APPWINDOW
     res = SetWindowLong(hwnd, GWL_EXSTYLE, GetWindowLong(hwnd, GWL_EXSTYLE) & ~WS_EX_TOOLWINDOW | WS_EX_APPWINDOW)
     # re-assert the new window style
     window.wm_withdraw()
-    window.after(10, lambda: window.wm_deiconify())
+    window.after(1, lambda: window.wm_deiconify())
 
 
-def window_embed(window: tk.Tk, toplevel: tk.Toplevel):
+def window_embed(window: tk.Tk, toplevel: tk.Toplevel, iswindow: bool = True):
     def embed():
         SetParent(GetParent(toplevel.winfo_id()), window.winfo_id())
-        window.attributes("-topmost", False)
-    window.attributes("-topmost", True)
-    window.after(0, embed)
+        toplevel.deiconify()
+        if iswindow:
+            window.attributes("-topmost", False)
+    if iswindow:
+        window.attributes("-topmost", True)
+    window.after(1, embed)
 
 
 def window_centre(window: tk.Tk):
@@ -285,6 +381,30 @@ class DevBorder(object):
             self.widget.geometry(f"{self.widget.winfo_width()}x{self.bottom.winfo_y()}")
 
 
+class DevStack(tk.Frame):
+    def __init__(self):
+        super(DevStack, self).__init__()
+        self._pages = {}
+
+    def add_page(self, page: tk.Widget, id: int = 0):
+        self._pages[id] = page
+
+    def show_page(self, id: int):
+        self._pages[id].pack(fill=tk.BOTH, expand=tk.YES)
+        for item in self._pages.keys():
+            if not item == id:
+                self.hide_page(item)
+                print("Hide" + str(item))
+            else:
+                print("Show" + str(item))
+
+    def hide_page(self, id: int):
+        self._pages[id].pack_forget()
+
+    def get_pages(self):
+        return self._pages
+
+
 class DevSideBar(tk.Frame):
     def __init__(self, master: tk.Widget, background="#ffffff", ):
         super(DevSideBar, self).__init__(master=master, background=background)
@@ -490,19 +610,19 @@ class DevTitleBar(tk.Frame):
 
     def add_close(self, close_bg="#ffffff", close_fg="#000000", close_active_bg="#e81123", close_active_fg="#f5f5f5",
                   close_func=None):
-        self.closebutton = tk.Button(self, text='×', borderwidth=0, background=close_bg, foreground=close_fg,
+        self.closebutton = tk.Button(self, text='×', borderwidth=0, background=close_bg, foreground=close_fg, font=("微软雅黑", 13),
                                      activebackground=close_active_bg, command=close_func,
                                      activeforeground=close_active_fg)
         self.closebutton.pack(fill=tk.Y, side=self.button_side, ipadx=5)
 
     def add_max(self, max_bg="#ffffff", max_fg="#000000", max_active_bg="#c2c2c2", max_active_fg="#ffffff"):
-        self.maxbutton = tk.Button(self, text="▢", borderwidth=0, background=max_bg, foreground=max_fg,
+        self.maxbutton = tk.Button(self, text="▢", borderwidth=0, background=max_bg, foreground=max_fg, font=("微软雅黑", 13),
                                    activebackground=max_active_bg,
                                    activeforeground=max_active_fg)
         self.maxbutton.pack(fill=tk.Y, side=self.button_side, ipadx=5)
 
     def add_min(self, min_bg="#ffffff", min_fg="#000000", min_active_bg="#c2c2c2", min_active_fg="#ffffff"):
-        self.minbutton = tk.Button(self, text="-", borderwidth=0, background=min_bg, foreground=min_fg,
+        self.minbutton = tk.Button(self, text="-", borderwidth=0, background=min_bg, foreground=min_fg, font=("微软雅黑", 13),
                                    activebackground=min_active_bg,
                                    activeforeground=min_active_fg)
         self.minbutton.pack(fill=tk.Y, side=self.button_side, ipadx=8)
@@ -516,17 +636,10 @@ class DevTitleBar(tk.Frame):
     def widget_max(self):
         if self.iswindow:
             if self.ismax:
-                self.window.geometry(f"{self._width}x{self._height}+{self._x}+{self._y}")
+                self.window.state("normal")
                 self.ismax = False
             elif not self.ismax:
-                self._x = self.window.winfo_x()
-                self._y = self.window.winfo_y()
-                self._width = self.window.winfo_width()
-                self._height = self.window.winfo_height()
-                self.window.geometry(f"{self.winfo_screenwidth()}x{self.winfo_screenheight()}+0+0")
-                self.window.attributes('-topmost', 1)
-                self.window.attributes('-topmost', 0)
-                self.window.geometry("+0+0")
+                self.window.state("zoomed")
                 self.ismax = True
             self.window.update()
         if not self.iswindow:
@@ -547,29 +660,12 @@ class DevTitleBar(tk.Frame):
 
     def widget_min(self):
         if self.iswindow:
-            windowsize = (self.window.winfo_width(), self.window.winfo_height())
-            self.window.withdraw()
-            self.minwindow = DevToplevel()
-            self.minwindow.configure(background=self.window.cget("background"))
-            self.minwindow.geometry(f"50x50+{windowsize[0]}+{windowsize[1]}")
-            self.minwindow.overrideredirect(True)
-            if self.minwindow_icon is None:
+            self.window.state("iconic")
+        elif not self.iswindow:
+            try:
+                self.widget.forget()
+            except tk.TclError:
                 pass
-            else:
-                self.image = tk.Label(self.minwindow, image=tk.PhotoImage(file=self.minwindow_icon))
-                self.image.pack(fill=tk.BOTH, expand=tk.YES, padx=5, pady=5)
-            resize = DevBorder(self.minwindow, border_color=self.minwindow_border_color)
-            add_taskbar(self.minwindow)
-            self.minwindow.attributes("-topmost", True)
-            DevDrag(self.minwindow, self.minwindow, iswindow=True)
-
-            def show_window(evt):
-                self.window.deiconify()
-                self.window.attributes("-topmost", True)
-                self.minwindow.destroy()
-                self.window.attributes("-topmost", False)
-
-            self.minwindow.bind("<Double-Button-1>", show_window)
 
     def show(self):
         self.pack(fill=tk.X, side=tk.TOP)
@@ -599,105 +695,45 @@ class DevToast(tk.Toplevel):
         self.attributes("-topmost", True)
 
 
-class DevToplevel(tk.Toplevel):
-    def __init__(self, master: tk.Tk = None):
-        super(DevToplevel, self).__init__(master=master)
-        self.title("")
-        self.geometry("630x300")
-        self.configure(background="#ffffff")
-        self.iconbitmap(Icon_Empty)
-
-    def wm_run(self):
-        try:
-            self.mainloop()
-        except tk.TclError:
-            return False
-        else:
-            return True
-
-    run = wm_run
-
-    def wm_statusbar(self, statusBar: tk.Widget = None):
-        if statusBar is None:
-            pass
-        else:
-            self._statusBar = statusBar
-            self._statusBar.pack(fill=tk.X, side=tk.BOTTOM)
-        return self._statusBar
-
-    statusbar = wm_statusbar
-
-    def wm_titlebar(self, titleBar: tk.Label = None, showtask: bool = True):
-        if titleBar is None:
-            self.minsize(100, 30)
-            self.overrideredirect(True)
-            if showtask:
-                self.after(10, lambda: window_add_taskbar(self))
-        else:
-            self._titlebar = titleBar
-            self._titlebar.pack(fill=tk.X, side=tk.TOP)
-            DevDrag(self._titlebar, self, True)
-        return self._titlebar
-
-    titlebar = wm_titlebar
-
-    def wm_menubar(self, menubar: DevMenuBar = None):
-        menubar.show()
-
-    menubar = wm_menubar
-
-    def wm_centre(self):
-        self.after(1, lambda evt=None: window_centre(self))
-
-    centre = wm_centre
-
-    def min_window(self):
-        self.withdraw()
-        self.minwindow = DevToplevel(self)
-        self.minwindow.overrideredirect(True)
-        self.minwindow.attributes("-topmost", True)
-        DevDrag(self.minwindow, self.minwindow, iswindow=True)
-        self.minwindow.bind("<Double-Button-1>", lambda evt: self.deiconify())
-
-
 class DevWindow(tk.Tk):
-    def __init__(self):
+    def __init__(self, title: str = ""):
         super(DevWindow, self).__init__()
-        self.title("")
+        self.title(title)
         self.geometry("630x300")
         self.configure(background="#ffffff")
         self.iconbitmap(Icon_Empty)
 
+    def wm_remove_titlebar(self):
+        self.after(0, lambda: window_custom_border_taskbar(self))
+
+    remove_titlebar = wm_remove_titlebar
+
+    def wm_remove_titlebar_none_border(self):
+        window_custom_taskbar(self)
+
+    remove_titlebar_none_border = wm_remove_titlebar_none_border
+
     def wm_run(self):
-        try:
-            self.mainloop()
-        except tk.TclError:
-            return False
-        else:
-            return True
+        self.mainloop()
 
     run = wm_run
 
     def wm_statusbar(self, statusBar: tk.Widget = None):
-        if statusBar is None:
-            pass
-        else:
-            self._statusBar = statusBar
-            self._statusBar.pack(fill=tk.X, side=tk.BOTTOM)
+        self._statusBar = statusBar
+        self._statusBar.pack(fill=tk.X, side=tk.BOTTOM)
         return self._statusBar
 
     statusbar = wm_statusbar
 
-    def wm_titlebar(self, titleBar: tk.Label = None, showtask: bool = True):
-        if titleBar is None:
-            self.minsize(100, 30)
-            self.overrideredirect(True)
-            if showtask:
-                self.after(10, lambda: window_add_taskbar(self))
-        else:
-            self._titlebar = titleBar
-            self._titlebar.pack(fill=tk.X, side=tk.TOP)
-            DevDrag(self._titlebar, self, True)
+    def wm_titlebar(self, titleBar: tk.Label = None, overtitlebar: bool = True, border: bool = True):
+        self.minsize(100, 30)
+        if overtitlebar:
+            window_add_taskbar(self)
+        if border:
+            self.remove_titlebar()
+        self._titlebar = titleBar
+        self._titlebar.pack(fill=tk.X, side=tk.TOP)
+        DevDrag(self._titlebar, self, True)
         return self._titlebar
 
     titlebar = wm_titlebar
@@ -713,17 +749,23 @@ class DevWindow(tk.Tk):
     centre = wm_centre
 
     def min_window(self):
-        self.withdraw()
-        self.minwindow = DevToplevel(self)
-        self.minwindow.overrideredirect(True)
-        self.minwindow.attributes("-topmost", True)
-        DevDrag(self.minwindow, self.minwindow, iswindow=True)
-        self.minwindow.bind("<Double-Button-1>", lambda evt: self.deiconify())
+        self.state("iconic")
+
+    def dpi(self):
+        self.tk.call('tk', 'scaling', ScaleFactor/75)
+
+
+class DevToplevel(tk.Toplevel, DevWindow):
+    def __init__(self, master: tk.Tk = None, title: str = ""):
+        super(DevToplevel, self).__init__(master=master)
+        self.title(title)
+        self.geometry("630x300")
+        self.configure(background="#ffffff")
+        self.iconbitmap(Icon_Empty)
 
 
 if __name__ == '__main__':
     Root = DevWindow()
-    Top = DevToplevel()
-    window_embed(Root, Top)
-    Root.centre()
+    TitleBar = DevTitleBar(Root, iswindow=True, window=Root)
+    Root.titlebar(TitleBar, overtitlebar=False)
     Root.run()
